package io

import portal._

import java.util.Calendar
import scala.io.Source

class Reader(private var _ufes: Universidade){

    def leCursos(arq: String) = {

        //ignora primeira linha
        val lines = Source.fromFile(arq).getLines.toList.drop(1)

        for (line <- lines) {
            val dados = line.split(";")

            _ufes.addCurso( new Curso(dados(0).toInt, dados(1)) )
            //println("added: " + dados(0) + " " + dados(1))
        }

    }

    def leDisciplinas(arq: String) = {

        //ignora primeira linha
        val lines = Source.fromFile(arq).getLines.toList.drop(1)

        for (line <- lines) {
            val dados = line.split(";")

            _ufes.addDisciplina( new Disciplina(dados(0), dados(1)) )
            //println("added: " + dados(0) + " " + dados(1))
        }
    
    }

    def leAvaliacoes(arq: String) = {

        //ignora primeira linha
        val lines = Source.fromFile(arq).getLines.toList.drop(1)

        for (line <- lines) {
            val dados = line.split(";")

            dados(4) match {
                case "P" | "F" => if(dados.size > 6) throw new IllegalArgumentException("Tamanho máximo de grupo especificado para a prova " + dados(1) + ": " + dados(6) + ".")
                case _ => ;
            }

            val dt = dados(5).split("/")
            val data = Calendar.getInstance()
            data.set((dt(2).toInt-1900).toInt, dt(1).toInt, dt(0).toInt)

            //println("added: " + dados(0) + " " + dados(1) + " " + dados(2) + " " + dados(3)
            //    + " " + dados(4) + " " + dados(5) + " " + data.get(Calendar.DAY_OF_MONTH) + data.get(Calendar.MONTH) + data.get(Calendar.YEAR))

            dados(4) match {
                case "P" | "F" => _ufes.addAvaliacao( new Prova(dados(1), dados(2), dados(3).toDouble, data, dados(4) == "F"), dados(0) )
                case "T" => _ufes.addAvaliacao( new Trabalho(dados(1), dados(2), dados(3).toDouble, data, dados(6).toInt), dados(0) )
                case _ => 
                    throw new IllegalArgumentException("Tipo de avaliação desconhecido para " + dados(1) + ": " + dados(4) + ".")
            }
        }

        _ufes.disciplinaSemAvaliacao match {
            case Some(d) => throw new IllegalStateException("A disciplina " + d.codigo + " não possui nenhuma avaliação cadastrada.")
            case _ => ;
        }
    
    }

    def leAlunos(arq: String) = {

        //ignora primeira linha
        val lines = Source.fromFile(arq).getLines.toList.drop(1)

        for (line <- lines) {
            val dados = line.split(";")

            var aluno = dados(3) match {
                case "G" => _ufes.buscaCurso(dados(4).toInt) match {
                    case Some(c) => new AlunoGraduacao(dados(0).toInt, dados(1),  c)
                    case _ => throw new IllegalArgumentException("Código de curso não definido usado no aluno " + dados(0) + " : " + dados(4))
                }
                case "P" => dados(4) match {
                    case "M" | "D" => new AlunoPosGraduacao(dados(0).toInt, dados(1),  dados(4)(0))
                    case _ => throw new IllegalArgumentException("Tipo de aluno de pós-graduação desconhecido para " + dados(0) + ": " + dados(4) + ".")
                }
                case _ => throw new IllegalArgumentException("Tipo de aluno desconhecido para " + dados(0) + ": " + dados(3) +".")
            }

            //Adiciona o aluno em cada disciplina em que esta matriculado
            for(d <- dados(2).split(","))
                _ufes.addAluno(aluno , d.trim() )

            //println("added: " + dados(0) + " " + dados(1) + " " + dados(2) + " " + dados(3) + " " + dados(4))
        }
    
    }

    def leNotas(arq: String) = {

        //ignora primeira linha
        val lines = Source.fromFile(arq).getLines.toList.drop(1)

        for (line <- lines) {
            val dados = line.split(";")

            var alunos = dados(1).split(",").toList
            alunos = alunos.map(x => x.trim())

            _ufes.buscaAvaliacao(dados(0)) match {
                case Some(a) => a.ehTrabalho match {
                    case true => a.asInstanceOf[Trabalho].addGrupo( fazGrupo(alunos, new Nota(dados(2).replaceAll(",", ".").toDouble, a)) )
                    case false => _ufes.buscaAlunoPorAvaliacao(alunos(0).toInt, a).addNota(new Nota(dados(2).replaceAll(",", ".").toDouble, a))
                }
                case _ => throw new IllegalArgumentException("Código de avaliação não definido usado na" + 
                    "planilha de notas, associado ao(s) aluno(s) " + dados(1) + " : " + dados(0))
            }

            //println("added: " + dados(0) + " " + dados(1) + " " + dados(2))
        }
    
    }

    def fazGrupo(alunos: List[String], nota: Nota): Grupo = {
        var g = new Grupo(nota)

        for(mat <- alunos)
            g.addAluno( _ufes.buscaAlunoPorAvaliacao(mat.toInt, nota.avaliacao) )

        return g
    }

}