package portal

class Universidade() {

	var _cursos = Map[Int, Curso]() //Codigo, Objeto Curso
	var _disciplinas = Map[String, Disciplina]() //Codigo, Objeto Disciplina

	//Getters e Setters
    def cursos = _cursos.values.toList

	def disciplinas = _disciplinas.values.toList

	def addCurso(c: Curso) = {

    	_cursos.get(c.codigo) match {
		  case Some(_) =>
		    throw new IllegalArgumentException("Código repetido para curso: " + c.codigo + ".")
		  case None =>
		    _cursos += (c.codigo -> c)
		}

    }

    def addDisciplina(d: Disciplina) = {

    	_disciplinas.get(d.codigo) match {
		  case Some(_) =>
		    throw new IllegalArgumentException("Código repetido para disciplina: " + d.codigo + ".")
		  case None =>
		    _disciplinas += (d.codigo -> d)
		}

    }

    def addAvaliacao(ava: Avaliacao, codDiscip: String) = {

    	_disciplinas.get(codDiscip) match {
			case Some(d) =>
				d.addAvaliacao(ava)
			case None =>
				throw new IllegalArgumentException("Código de disciplina não definido usado na avalição " + ava.codigo + ": " + codDiscip + ".")
		}

    }

    def addAluno(a: Aluno, codDiscip: String) = {

    	_disciplinas.get(codDiscip) match {
			case Some(d) =>
				d.AddAluno(a)
			case None =>
				throw new IllegalArgumentException("Código de disciplina não definido usado no aluno " + a.matricula + ": " + codDiscip + ".")
		}

    }

    def buscaAvaliacao(codAva: String): Option[Avaliacao] = {

    	for((c, d) <- _disciplinas)
    		d.getAvaliacao(codAva) match {
                case Some(ava) => return Some(ava)
                case _ => ;
            }

        return None

    }

    def disciplinaSemAvaliacao = _disciplinas.values.find(x => x.qtdAvaliacoes == 0)

    def buscaCurso(codCurso: Int) = _cursos.get(codCurso)

    def buscaAlunoPorAvaliacao(mat: Int, ava: Avaliacao): Aluno = {
    	for((c, d) <- _disciplinas)
    		d.getAvaliacao(ava.codigo) match {
                case Some(_) => d.getAluno(mat) match {
                	case Some(a) => return a
                	case _ => ;
                }
                case _ => ;
            }

        throw new IllegalArgumentException("Matrícula de aluno não definida usada na planilha de" + 
        	"notas, associada à avaliação " + ava.codigo + " : " + mat + ".")
    }
}